package com.tistory.antop.proftpd.aop;

import java.util.Map.Entry;

import org.apache.log4j.Logger;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.authentication.AuthenticationCredentialsNotFoundException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.ui.ModelMap;

import com.tistory.antop.proftpd.common.exception.BizException;

/**
 * 컨트롤러단의 첫번째 인자가 ModelMap 인 메소드는 AJAX 메소드이다.
 * 응답에 success와 message를 추가한다.
 * 
 * @author Antop
 * 
 */
public class AjaxAdvice {

	private Logger logger = Logger.getLogger(getClass());

	public ModelMap around(ProceedingJoinPoint joinPoint) {

		BizException exception = null;
		ModelMap modelMap = new ModelMap();

		// 입력된 데이터 로그
		if (logger.isDebugEnabled()) {
			logger.debug("before - " + getMethodInfo(joinPoint));

			Object[] args = joinPoint.getArgs();
			int size = args.length;

			// index 0은 ModelMap 이므로 출력 안해도 된다.
			for (int i = 1; i < size; i++) {
				logger.debug("args[" + i + "] = " + args[i]);
			}
		}

		try {
			// 프로세스 실행
			modelMap = (ModelMap) joinPoint.proceed();

			// ModelMap 으로 리턴된 데이터 로그
			if (logger.isDebugEnabled()) {
				logger.debug("after returning - " + getMethodInfo(joinPoint));
				for (Entry<String, Object> entry : modelMap.entrySet()) {
					logger.debug(entry.getKey() + " = " + entry.getValue());
				}
			}

		} catch (AuthenticationCredentialsNotFoundException e) {
			// 로그인이 안된 상태
			exception = new BizException("L", e.getMessage());
		} catch (AccessDeniedException e) {
			// 익명 사용자일 경우 로그인 요청
			Authentication auth = SecurityContextHolder.getContext()
					.getAuthentication();
			
			// 익명 사용자일 경우 로그인이 안된 상태
			if (auth.getName().equals("anonymousUser")) {
				// 에러코드에 "L" 값을 넘긴다.
				exception = new BizException("L", e.getMessage());
			} else {
				// 권한 없음
				exception = new BizException("권한이 없습니다.");
			}
		} catch (Throwable t) {
			// 그밖에 에러
			logger.error("after throwing - " + getMethodInfo(joinPoint), t);

			exception = new BizException(t.getMessage());
			exception.setParent(t);
		}

		if (exception != null) {
			modelMap.put("success", false);
			modelMap.put("error", exception);
		} else {
			modelMap.put("success", true);
		}

		return modelMap;
	}

	private String getMethodInfo(JoinPoint joinPoint) {
		return joinPoint.getSignature().toString();
	}

}