package com.tistory.antop.proftpd.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.context.SecurityContextRepository;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import com.tistory.antop.proftpd.common.BaseController;
import com.tistory.antop.proftpd.common.command.ListCommand;
import com.tistory.antop.proftpd.common.exception.BizException;
import com.tistory.antop.proftpd.common.vo.InquireOutVo;
import com.tistory.antop.proftpd.controller.command.LoginCommand;
import com.tistory.antop.proftpd.controller.command.UserInGroupListCommand;
import com.tistory.antop.proftpd.controller.command.UserSaveCommand;
import com.tistory.antop.proftpd.mybatis.model.User;
import com.tistory.antop.proftpd.mybatis.model.UserInGroup;
import com.tistory.antop.proftpd.service.UserSo;
import com.tistory.antop.proftpd.service.vo.UserInGroupInquireVo;
import com.tistory.antop.proftpd.service.vo.UserInquireVo;
import com.tistory.antop.proftpd.service.vo.UserSaveVo;

@Controller
public class UserController extends BaseController {

	@Autowired
	private UserSo userSo;

	/**
	 * 사용자 목록 조회
	 * @param modelMap
	 * @param command
	 * @return
	 */
	@RequestMapping(value = "/user/list", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap list(ModelMap modelMap, @RequestBody ListCommand command) {

		UserInquireVo inVo = new UserInquireVo();

		mapping(command, inVo);

		InquireOutVo<User> outVo = userSo.inquire(inVo);

		// 비밀번호를 비워서 내보낸다.
		for(User user : outVo.getList()) {
			user.setPasswd(null);
		}
		
		modelMap.put("results", outVo.getList());
		modelMap.put("total", outVo.getTotal());

		return modelMap;
	}

	/**
	 * 사용자 정보 저장
	 * @param modelMap
	 * @param command
	 * @return
	 */
	@RequestMapping(value = "/user/save", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap save(ModelMap modelMap, @RequestBody UserSaveCommand command) {

		UserSaveVo saveVo = new UserSaveVo();

		mapping(command, saveVo);

		User user = userSo.save(saveVo);

		modelMap.put("user", user);

		return modelMap;
	}

	/**
	 * 사용자 삭제
	 * @param modelMap
	 * @param ids
	 * @return
	 */
	@RequestMapping(value = "/user/remove", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap remove(ModelMap modelMap, @RequestBody List<Long> ids) {

		userSo.remove(ids);

		return modelMap;
	}

	/**
	 * 
	 * @param command 화면에서 넘어온 사용자 커맨드 객체
	 * @param saveVo 서비스단으로 넘겨줄 저장할 사용자 객체
	 */
	private void mapping(UserSaveCommand command, UserSaveVo saveVo) {
		saveVo.setId(command.getId());
		saveVo.setUserid(command.getUserid());
		saveVo.setPasswd(command.getPasswd());
		saveVo.setEmail(command.getEmail());
		saveVo.setValid(command.getValid());
		saveVo.setAuthority(command.getAuthority());
	}

	/**
	 * 그룹에 속한 유저의 목록 조회
	 * @param modelMap
	 * @param command
	 * @return
	 */
	@RequestMapping(value = "/member/list/group", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap listInGroup(ModelMap modelMap,
			@RequestBody UserInGroupListCommand command) {

		UserInGroupInquireVo inVo = new UserInGroupInquireVo();

		mapping(command, inVo);
		inVo.setGroupName(command.getGroupName());

		InquireOutVo<UserInGroup> outVo = userSo.inquireByGroup(inVo);

		modelMap.put("results", outVo.getList());
		modelMap.put("total", outVo.getTotal());

		return modelMap;
	}

	/**
	 * 스프링 시큐리티 인증 관리
	 */
	@Autowired
	@Qualifier("authenticationManager")
	private AuthenticationManager authenticationManager;
	@Autowired
	private SecurityContextRepository repository;

	/**
	 * 로그인
	 * @param modelMap
	 * @param request
	 * @param response
	 * @param command 아이디 / 비밀번호
	 * @return
	 */
	@RequestMapping(value = "/login", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap login(ModelMap modelMap, HttpServletRequest request,
			HttpServletResponse response, @RequestBody LoginCommand command) {

		String username = command.getUsername();
		String password = command.getPassword();

		// 토큰 생성
		UsernamePasswordAuthenticationToken token = new UsernamePasswordAuthenticationToken(
				username, password);

		try {
			// 인증 (실패시 BadCredentialsException)
			Authentication auth = authenticationManager.authenticate(token);
			
			// 로그인 정보 등록
			SecurityContextHolder.getContext().setAuthentication(auth);
			repository.saveContext(SecurityContextHolder.getContext(), request,
					response);

			modelMap.put("auth", auth);
		} catch (BadCredentialsException e) {
			throw new BizException("인증정보가 불일치합니다.");
		}

		return modelMap;
	}

	/**
	 * 로그아웃
	 * @param modelMap
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping(value = "/logout")
	@ResponseBody
	public ModelMap logout(ModelMap modelMap, HttpServletRequest request,
			HttpServletResponse response) {

		SecurityContextHolder.getContext().setAuthentication(null);
		repository.saveContext(SecurityContextHolder.getContext(), request,
				response);

		return modelMap;
	}
}
